import { 
  select, 
  csv, 
  timeParse, 
  timeFormat, 
  scaleLinear, 
  scaleTime,
  extent, 
  max,
  scaleBand,
  axisLeft,
  axisRight,
  axisTop,
  line,
  zoom,
  event,
  brush
} from 'd3';


const svg = select('svg');
const g = svg.append('g');
const width = +svg.attr('width');
const height = +svg.attr('height');
const margin = { top: 100, bottom: 20, left: 50, right: 50 };
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.bottom - margin.top;
const gEl = g.append('g')
  .attr('class', 'container')
  .attr('transform', `translate(${margin.left}, ${margin.top})`);

const clip = svg.append("defs").append("svg:clipPath");
clip
      .attr("id", "clip")
      .append("g:rect")
      .attr("width", innerWidth )
      .attr("height", innerHeight )
      .attr("x", 0)
      .attr("y", 0);

const parseDate = timeParse("%-d/%-m/%Y");
const formatDate = timeFormat("%b %Y");



let data;
let dataL;
let brushing = brush();
let zooming = zoom();
let container = select('#container');

const render = () => {
  
  const xValueB = d => d.Date;
  const yValueB = d => d.EffectiveRainfall;
  const yValueL = d => d.WaterLevel;
  const circleRadius = 5;
  

  const xAxisBTickFormat = data => formatDate(data);
  
  const xScaleB = scaleTime()
  .domain(extent(data, xValueB))
  .range([0, innerWidth])
  .nice();
  

  const yScaleB = scaleLinear()
    .domain([0, max(data, yValueB)])
    .range([0,innerHeight])
  
  const yScaleL = scaleLinear()
    .domain([0, max(data, yValueL)])
    .range([0,innerHeight])
  

  const yAxisB = axisLeft(yScaleB);
  const yAxisL = axisRight(yScaleL);

  const xAxisB = axisTop(xScaleB)
    .tickFormat(xAxisBTickFormat);
   
  const yAxisBGroup = gEl.append('g');
  yAxisBGroup.call(yAxisB);
  
  const yAxisLGroup = gEl.append('g');
  yAxisLGroup.call(yAxisL)
    .attr('transform', `translate(${innerWidth},0)`);
  
  
  const xAxisBGroup = gEl.append('g').call(xAxisB);
  xAxisBGroup
    .attr('transform', `translate(0, -2)`)
    .merge(xAxisBGroup)
    .selectAll("text")
      .style("text-anchor", "end")
        .attr("dx", "-.8em")
        .attr("dy", ".15em")
        .attr("transform", "rotate(65)");
   
  
//render(gEl, {data});
  
  zooming.on('zoom', function(){
    let newXScale = event.transform.rescaleX(xScaleB);
    let newYBScale = event.transform.rescaleY(yScaleB);
    let newYLScale = event.transform.rescaleY(yScaleL);
    
      const render = (selection, {data, dataL}) => {

    xAxisB.scale(newXScale)
    xAxisBGroup.call(xAxisB)
    yAxisB.scale(newYBScale)
    yAxisBGroup.call(yAxisB)
    yAxisL.scale(newYLScale)
    yAxisLGroup.call(yAxisL)
      const rects = selection.selectAll('rect').data(data);
  rects
    .enter()
    .append('rect')
      .attr("clip-path", "url(#clip)")
      .attr('fill','steelblue')
      .attr('width', 5)
      .attr('x', d => xScaleB(xValueB(d)))
      .attr('height', d => yScaleB(yValueB(d)));
    rects
      .attr('x', d => newXScale(xValueB(d)))
      .attr('height', d => newYBScale((yValueB(d))));
  
      const circles = selection.selectAll('circle').data(dataL);
    circles
      .enter()
      .append('circle')
        .attr("clip-path", "url(#clip)")
        .attr('cx', d => xScaleB(xValueB(d)))
        .attr('cy', d => yScaleL(yValueL(d)))
        .attr('r', 5);
    circles
      .attr('cx', d => newXScale(xValueB(d)))
      .attr('cy', d => newYLScale((yValueL(d))));
        
   const lineGenerator = line()
   .x(d => xScaleB(xValueB(d)))
   .y(d => yScaleL(yValueL(d)));
        
   const lineGeneratorU = line()
   .x(d => newXScale(xValueB(d)))
   .y(d => newYLScale(yValueL(d)));
     
     
      const linePath = selection.selectAll('.line-path').data(dataL);
    linePath
      .enter()
      .append('path')
        .attr("clip-path", "url(#clip)")
        .attr('class', 'line-path')
        .attr('d',lineGenerator(dataL)); 

        
    linePath
      .attr('class', 'line-path')
      .attr('d',lineGeneratorU(dataL)); 
  } 
    
    render(gEl, {data, dataL});
    // selectAll('.rectsE').data(data)
    // .enter()
    // .append('rect')
    //   .attr('class','rectsE')
    //   .attr('width', 5)
    //   .attr('x', d => xScaleB(xValueB(d)))
    //   .attr('height', d => yScaleB(yValueB(d)))
    
  })
svg.call(zooming);
   
};

Promise
  .all([
    csv('https://vizhub.com/careyshan/datasets/line-bar-chart-edit.csv'),
    csv('https://vizhub.com/careyshan/datasets/line-chart1.csv')
  ])
  .then(([loadedDataBar,loadedDataLine]) => {
    data = loadedDataBar;
    dataL = loadedDataLine;
    data.forEach(d => {
      d.Date = parseDate(d.Date),
      d.WaterLevel = +d.WaterLevel,
      d.EffectiveRainfall = +d.EffectiveRainfall
    });
    dataL.forEach(d => {
    d.Date = parseDate(d.Date),
    d.WaterLevel = +d.WaterLevel,
    d.EffectiveRainfall = +d.EffectiveRainfall
    })
  render(gEl, {data});
});



// csv('https://vizhub.com/careyshan/datasets/line-bar-chart-edit.csv')
//   .then(loadedData => {
//     data = loadedData;
//     data.forEach(d => {
//       d.Date = parseDate(d.Date),
//       d.WaterLevel = +d.WaterLevel,
//       d.EffectiveRainfall = +d.EffectiveRainfall
//     });
//   render();

//   });

